<?php
require_once("assertionChecking.interface.php");
require_once 'policy.class.php';
    /**
     * Class that checks if a given sir assertion is valid.
     */
    class sirAssertionChecking implements IAssertionChecking {
        private $assertion;
        private $policy;
        private $personId;
        private $error;

        /**
         * Load the policies from a  given file.
         * @param <type> $rulesf The archive file with the policy.
         */
        public function __construct($rulesf = "config/policies.xml") {
             $this->error = false;
            if(0==sizeof($this->getPolicy($rulesf))) {
                error_log("ERROR EXTRAYENDO POLÍTICAS");
                $this->error = true;
            }
        }

        /**
         * Function that checks if the assertion is authorized
         * @param array $assertion Assertion to check
         * @return bool. True if the policy matches the assertion.
         */
        public function checkAssertion($assertion) {
         //   error_log("sirAssertionChecking.checkAssertion");
       //     $this->assertion = $this->transformAssertion($assertion);
        // Sin array
            $this->assertion = $assertion;
            $dev = false;
            if($this->matchRules()!=false) {
                $this->personId = $this->assertion['sPUC'];           
                $dev = true;
            }
            return $dev;
        }
        /**
         * Function that decides if the assertion matches the policy,
         * @param <type> $assertion
         * @return bool. True if the policy matches the assertion.
         */
        protected function matchRules() {
      //      error_log("sirAssertionChecking.matchRules");
            $dev = true;
            if(sizeof($this->policy)>0) {
                foreach ($this->policy as $pol) {
                    $dev = $dev && $pol->checkPolicy($this->assertion);
                }
            }else {
                $dev = false;
            }
            return $dev;
        }

            /**
         * Function that loads the Policy from a given file
         * @param <type> $rulesfile File of the policy
         * @return AssertionPolicy. The Policy.
         */
        protected function getPolicy($rulesfile) {
      //      error_log('sirAssertionChecking.getPolicy');
            $this->policy=array();
            $xml = simplexml_load_file($rulesfile);
            if(strcmp($xml->getName(),"AssertionList")==0) {
                $i=0;
                while($i<sizeof($xml->children())) {
                    $arr = $xml->children();
                    if(strcmp("sir",$arr[$i]['type'])==0) {
                        $j=0;
                        while($j<sizeof($arr[$i]->Policies->children())) {
                            $arr2 =  $arr[$i]->Policies->Policy;
                            $this->policy[$j] = new AssertionPolicy($arr2[$j]);
                            $j++;
                        }
                    }
                    $i++;
                }
            }
            return $this->policy;
        }


                   /**
         * Person Id getter
         * @return String: The scope.
         */
        public function getPersonId() {
  //          error_log("sirAssertionChecking.getPersonId");
            return $this->personId[0];
        }
       /**
         * Function that transform the asserttion in a compatible format to be checked
         * @param string $assertion: saml2 assertion
         * @return Array(sHO=> X, sPUC=>Y,ePA=>Z)
         */
        private function transformAssertion($assertion) {
        //    error_log('sirAssertionChecking.transformAssertion:');
            $transform = array();
            $aux = explode(",",$assertion);
            foreach($aux as $a){
                $aux2= explode('=',$a);
                $transform[$aux2[0]] = $aux2[1];
            }
            foreach ($transform as $key => $value) {
                $transform[$key] = $this->manageMultipleAttrs($value);
            }
            return $transform;
        }

        /**
         * Function that manage multivaluated attributes.
         * @param String $value
         * @return String or Array
         */
        private function manageMultivaluatedAttrs($value) {
        //    error_log('sirAssertionChecking.manageMultivaluatedAttrs: '.print_r($value,true));
            if(strstr($value,"|")) {
                $value = explode("|", $value);
            }
            return $value;
        }
        /**
         * Function that manage Multiple attributes.
         * @param String $value
         * @return Array
         */
        private function manageMultipleAttrs($value) {
          // error_log('sirAssertionChecking.manageMultipleAttrs: ');
            if(strstr($value,",")) {
                $value = explode(",", $value);
                $arrAux = array();
                foreach ($value as $k => $v) {
                    $arrayAux2 = array();
                    $arrAux = explode('=',$v);
                    $arrayAux2[$arrAux[0]] = $this->manageMultivaluatedAttrs($arrAux[1]);
                    $value[$k] = $arrayAux2;
                }
            }else {
                $value = array($this->manageMultivaluatedAttrs($value));
            }
            return $value;
        }


            public function getError() {
            return $this->error;
        }

         

    }
    ?>